# Generated by roxygen2: do not edit by hand

S3method(bread,glmrob)
S3method(bread,lmrob)
S3method(bread,teeMod)
S3method(confint,teeMod)
S3method(estfun,glmrob)
S3method(estfun,lmrob)
S3method(estfun,teeMod)
S3method(getCall,teeMod)
S3method(lmitt,formula)
S3method(lmitt,lm)
S3method(print,summary.StudySpecification)
S3method(print,summary.teeMod)
S3method(summary,StudySpecification)
S3method(summary,teeMod)
S3method(update,teeMod)
S3method(vcov,teeMod)
export("blocks<-")
export("clusters<-")
export("forcings<-")
export("treatment<-")
export("unitids<-")
export("units_of_assignment<-")
export(a.)
export(adopters)
export(as.SandwichLayer)
export(as.lmitt)
export(as.teeMod)
export(as_obs_spec)
export(as_rct_spec)
export(as_rd_spec)
export(assigned)
export(atc)
export(ate)
export(ato)
export(att)
export(block)
export(blocks)
export(cluster)
export(clusters)
export(cov_adj)
export(default)
export(etc)
export(ett)
export(forcing)
export(forcings)
export(get_structure)
export(has_binary_treatment)
export(has_blocks)
export(identical_StudySpecifications)
export(identify_small_blocks)
export(lmitt)
export(obs_spec)
export(obs_specification)
export(obsstudy_spec)
export(obsstudy_specification)
export(olw)
export(owt)
export(pwt)
export(rct_spec)
export(rct_specification)
export(rd_spec)
export(rd_specification)
export(specification_data_concordance)
export(specification_table)
export(stable)
export(treatment)
export(unit_of_assignment)
export(unitid)
export(unitids)
export(units_of_assignment)
export(uoa)
export(var_names)
export(var_table)
export(vcov_tee)
export(z.)
exportMethods("*")
exportMethods("+")
exportMethods("-")
exportMethods("/")
exportMethods("[")
exportMethods("blocks<-")
exportMethods("clusters<-")
exportMethods("forcings<-")
exportMethods("treatment<-")
exportMethods("unitids<-")
exportMethods("units_of_assignment<-")
exportMethods(blocks)
exportMethods(c)
exportMethods(clusters)
exportMethods(forcings)
exportMethods(show)
exportMethods(subset)
exportMethods(treatment)
exportMethods(unitids)
exportMethods(units_of_assignment)
exportMethods(weights)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,validObject)
importFrom(sandwich,bread)
importFrom(sandwich,estfun)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,cov)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.action)
importFrom(stats,na.pass)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,summary.lm)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(stats,weights)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,getS3method)
importFrom(utils,methods)
